#include "mlpproblem.h"

/** edo einai i synartisi dimioyrgias **/
MlpProblem::MlpProblem(int n,Dataset *d)
    :Problem((d->dimension()+2)*n)
{
    data = d;
    int k = (d->dimension()+2)*n;
    left.resize(k);
    right.resize(k);
    /** edo bazo ta oria ton
     *  parametron tou neuronikou diktyou **/
    for(int i=0;i<k;i++)
    {
        left[i]=-10;
        right[i]=10;
    }
    weight.resize(k);
}

void    MlpProblem::setWeights(Data &w)
{
    weight  =w ;
}

/** edo ypologizoume to train error pou einai
 *  kai i timi pou elaxistopoioume **/
double MlpProblem::funmin(Data &x)
{
    weight  =x ;
    return getTrainError();

}

/** edo epistrefoume tin paragogo tis
 *  synartisis funmin(x) os pros x**/
Data    MlpProblem::gradient(Data &x)
{
    Data g;
    weight = x;
    g.resize(weight.size());
    for(int i=0;i<g.size();i++)
        g[i]=0.0;
    for(int i=0;i<data->count();i++)
    {
        Data xx = data->getXPoint(i);
        Data gtemp = getDerivative(xx);
        double per=getOutput(xx)-data->getYPoint(i);
        for(int j=0;j<g.size();j++)	g[j]+=gtemp[j]*per;
    }
    for(int j=0;j<x.size();j++) g[j]*=2.0;
    return g;
}

double  MlpProblem::sig(double x)
{
    return 1.0/(1.0+exp(-x));
}

double  MlpProblem::sigder(double x)
{
    double s = sig(x);
    return s*(1.0-s);
}

/** einai i exodos tou neuronikou gia to protypo x**/
double  MlpProblem::getOutput(Data  &x)
{
    double arg=0.0;
    double per=0.0;
    int nodes = weight.size()/(data->dimension()+2);
     int d = data->dimension();
    for(int i=1;i<=nodes;i++)
    {
        arg=0.0;
        for(int j=1;j<=d;j++)
        {
            int pos=(d+2)*i-(d+1)+j-1;
            arg+=weight[pos]*x[j-1];
        }
        arg+=weight[(d+2)*i-1];
        per+=weight[(d+2)*i-(d+1)-1]*sig(arg);
    }
    return per;
}

/** einai i paragogos tou neuronikou os
 *  pros to protypo x**/
Data    MlpProblem::getDerivative(Data &x)
{
    double arg;
        double f,f2;
        int nodes = weight.size()/(data->dimension()+2);
        int d = data->dimension();
        Data G;
        G.resize(weight.size());

        for(int i=1;i<=nodes;i++)
        {
                arg = 0.0;
                for(int j=1;j<=d;j++)
                {
                        arg+=weight[(d+2)*i-(d+1)+j-1]*x[j-1];
                }
                arg+=weight[(d+2)*i-1];
                f=sig(arg);
                f2=f*(1.0-f);
                G[(d+2)*i-1]=weight[(d+2)*i-(d+1)-1]*f2;
                G[(d+2)*i-(d+1)-1]=f;
                for(int k=1;k<=d;k++)
                {
                        G[(d+2)*i-(d+1)+k-1]=
                                x[k-1]*f2*weight[(d+2)*i-(d+1)-1];
                }
    }
        return G;
}

/** edo exoume to sfalma ekpaideysis **/

double  MlpProblem::getTrainError()
{
    double error = 0.0;
    for(int i=0;i<data->count();i++)
    {
        Data xx = data->getXPoint(i);
        double yy = data->getYPoint(i);
        double per = getOutput(xx);
        error+= (per-yy)*(per-yy);
    }
    return error;
}

/** kanei oti kai i getTrainError() alla gia to test set **/
double  MlpProblem::getTestError(Dataset *test)
{
    double error = 0.0;
    for(int i=0;i<test->count();i++)
    {
        Data xx = test->getXPoint(i);
        double yy = test->getYPoint(i);
        double per = getOutput(xx);
        error+= (per-yy)*(per-yy);
    }
    return error;
}

/** edo epistrefo to classification sfalma sto test set **/
double  MlpProblem::getClassTestError(Dataset *test)
{
    double error = 0.0;
    for(int i=0;i<test->count();i++)
    {
        Data xx = test->getXPoint(i);
        double realClass = test->getClass(i);
        double per = getOutput(xx);
        double estClass = test->getClass(per);
        /** an apexei i pragmatiki klasi apo tin ypologismeni
         *  pano apo 10^-5, tote exoume sfalma **/
        error+= (fabs(estClass - realClass)>1e-5);
    }
    /** to metatrepoume se pososto **/
    return error*100.0/test->count();
}

MlpProblem::~MlpProblem()
{
    //nothing here
}
